# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['docile', 'docile.cli', 'docile.dataset', 'docile.evaluation', 'docile.tools']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.0,<9.0.0',
 'networkx[default]>=2.8.8,<3.0.0',
 'pdf2image>=1.16.0,<2.0.0',
 'pillow>=9.1.0,<10.0.0',
 'tabulate>=0.9.0,<0.10.0',
 'tqdm>=4.64.1,<5.0.0']

extras_require = \
{'interactive': ['jupyterlab>=3.5.0,<4.0.0',
                 'ipywidgets>=8.0.4,<9.0.0',
                 'plotly>=5.12.0,<6.0.0'],
 'ocr': ['python-doctr>=0.6.0,<0.7.0',
         'torch>=1.13.1,<2.0.0',
         'torchvision>=0.14.1,<0.15.0',
         'opencv-python>=4.6.0.66,<5.0.0.0']}

entry_points = \
{'console_scripts': ['docile_evaluate = docile.cli.evaluate:evaluate',
                     'docile_print_evaluation_report = '
                     'docile.cli.evaluate:print_evaluation_report']}

setup_kwargs = {
    'name': 'docile-benchmark',
    'version': '0.2.2',
    'description': 'Tools to work with the DocILE dataset and benchmark',
    'long_description': '# DocILE: Document Information Localization and Extraction Benchmark\n[![Tests](https://github.com/rossumai/docile/actions/workflows/tests.yml/badge.svg?branch=main)](https://github.com/rossumai/docile/actions/workflows/tests.yml)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n\nRepository to work with the [DocILE dataset and benchmark](https://docile.rossum.ai/), used in the DocILE\'23 CLEF Lab and ICDAR Competition.\nThe competition deadline is on May 10, 2023 and comes with a **$9000 prize pool**.\n\nThe repository consists of:\n* A python library, `docile`, making it easy to load the dataset, work with its annotations, pdfs and pre-computed OCR and run the evaluation.\n* An interactive [dataset browser notebook](docile/tools/dataset_browser.ipynb) to visualize the document annotations, predictions and evaluation results.\n* Baseline methods (will appear soon).\n\nTable of Contents:\n* [Download the dataset](#download-the-dataset)\n* [Installation](#installation)\n* [Predictions format and running evaluation](#predictions-format-and-running-evaluation)\n* [Pre-computed OCR](#pre-computed-ocr)\n* [Development instructions](#development-instructions)\n* [Dataset and benchmark paper](#dataset-and-benchmark-paper)\n\nAlso check [Tutorials](tutorials/) to get quickly started with the repo.\n\n## Download the dataset\n\nFirst you need to obtain a secret token by following the instructions at https://docile.rossum.ai/. Then download and unzip the dataset by running:\n```bash\n./download_dataset.sh TOKEN annotated-trainval data/docile --unzip\n./download_dataset.sh TOKEN synthetic data/docile --unzip\n./download_dataset.sh TOKEN unlabeled data/docile --unzip\n```\n\nRun `./download_dataset.sh --help` for more options, including how to only show urls (to download\nwith a different tool than curl), how to download smaller unlabeled/synthetic chunks or unlabeled\ndataset without pdfs (with pre-computed OCR only).\n\nYou can also work with the zipped datasets when you turn off image caching (check [Load and sample dataset](tutorials/load_and_sample_dataset.md) tutorial for details).\n\n## Installation\n\n### Option 1: Install as a library\nInstall the library with:\n```shell\npip install docile-benchmark\n```\n\nTo convert pdfs into images, the library uses https://github.com/Belval/pdf2image. On linux you might need to install:\n```shell\napt install poppler-utils\n```\nAnd on macOS:\n\n```shell\nbrew install poppler\n```\n\nNow you have all dependencies to work with the dataset annotations, pdfs, pre-comptued OCR and to run the evaluation. You can install extra dependencies by running the following (although using one of the provided dockerfiles, as explained below, might be easier in this case):\n```shell\npip install "docile-benchmark[interactive]"\npip install "docile-benchmark[ocr]"\n```\n\nThe first line installs additional dependencies allowing you to use the interactive dataset browser in [docile/tools/dataset_browser.py](docile/tools/dataset_browser.py) and the [tutorials](tutorials/). The second line let\'s you rerun the OCR predictions from scratch (e.g., if you\'d like to run it with different parameters) but to make it work, you might need additional dependencies on your system. Check https://github.com/mindee/doctr for the installation instructions (for pytorch).\n\n### Option 2: Use docker\nThere are two Dockerfiles available with the dependencies preinstalled:\n\n* `Dockerfile` is a lighter, CPU-only, version with all necessary dependencies to use the dataset with the pre-computed OCR and interactive browser.\n* `Dockerfile.gpu` has CUDA GPU support and contains additional dependencies needed to recompute OCR predictions from scratch (not needed for standard usage).\n\nYou can use docker compose to manage the docker images. First update the settings in `docker-compose.yml` and the port for jupyter lab in `.env`. Then build the image with:\n```shell\ndocker compose build jupyter[-gpu]\n```\nwhere `jupyter` uses `Dockerfile` and `jupyter-gpu` uses `Dockerfile.gpu`. You can then start the jupyter server:\n```shell\ndocker compose up -d jupyter[-gpu]\n```\n\nJupyter lab can be then accessed at `https://127.0.0.1:${JUPYTER_PORT}` (retrieve the token from logs with `docker compose logs jupyter[-gpu]`). You can also login to the container with:\n```shell\ndocker compose exec jupyter bash\n```\nAfter that run `poetry shell` to activate the virtual environment with the `docile` library and its dependencies installed.\n\n## Predictions format and running evaluation\n\nTo evaluate predictions for tasks KILE or LIR, use the following command:\n```bash\ndocile_evaluate \\\n  --task LIR \\\n  --dataset-path path/to/dataset[.zip] \\\n  --split val \\\n  --predictions path/to/predictions.json \\\n  --evaluate-x-shot-subsets "0,1-3,4+" \\  # default, show evaluation for 0-shot, few-shot and many-shot layout clusters\n  --evaluate-synthetic-subsets \\  # optional, show evaluation on layout clusters with available synthetic data\n  --evaluate-fieldtypes \\  # optional, show breakdown per fieldtype\n  --evaluate-also-text \\  # optional\n  --store-evaluation-result LIR_val_eval.json  # optional, it can be loaded in the dataset browser\n```\n\nRun `docile_evaluate --help` for more information on the options. You can also run `docile_print_evaluation_report --evaluation-result-path LIR_val_eval.json` to print the results of a previously computed evaluation.\n\nPredictions need to be stored in a single json file (for each task separately) containing a mapping from `docid` to the predictions for that document, i.e.:\n```json\n{\n    "docid1": [\n        {\n            "page": 0,\n            "bbox": [0.2, 0.1, 0.4, 0.5],\n            "fieldtype": "line_item_order_id",\n            "line_item_id": 3,\n            "score": 0.8,\n            "text": "Order 38",\n            "use_only_for_ap": true\n        },\n        "..."\n    ],\n    "docid2": [{"...": "..."}, "..."],\n    "..."\n}\n```\nExplanation of the individual fields of the predictions:\n  * `page`: page index (from zero) the prediction belongs to\n  * `bbox`: relative coordinates (from 0 to 1) representing the `left`, `top`, `right`, `bottom` sides of the bbox, respectively\n  * `fieldtype`: the fieldtype (sometimes called category or key) of the prediction\n  * `line_item_id`: ID of the line item. This should be a different number for each line item, the order does not matter. Omit for KILE predictions.\n  * `score` [optional]: the confidence for this prediction, can be omitted (in that case predictions are taken in the order in which they are stored in the list)\n  * `text` [optional]: text of the prediction, evaluated in a secondary metric only (when `--evaluate-also-text` is used)\n  * `use_only_for_ap` [optional, default is False]: only use the prediction for AP metric computation, not for f1, precision and recall (useful for less confident predictions).\n\nYou can use `docile.dataset.store_predictions` to store predictions represented with the `docile.dataset.Field` class to a json file with the required format.\n\n## Pre-computed OCR\n\nPre-computed OCR is provided with the dataset. The prediction was done using the [DocTR](https://github.com/mindee/doctr) library. On top of that, word boxes were snapped to text (check the code in [docile/dataset/document_ocr.py](docile/dataset/document_ocr.py)). These snapped word boxes are used in evaluation (description of the evaluation is coming soon).\n\nWhile this should not be needed, it is possible to (re)generate OCR from scratch (including the snapping) with the provided `Dockerfile.gpu`. Just delete `DATASET_PATH/ocr` directory and then access the ocr for each document and page with `doc.ocr.get_all_words(page, snapped=True)`.\n\n## Development instructions\n\nFor development, install [poetry](https://python-poetry.org/docs/) and run `poetry install`. Start a shell with the virtual environment activated with `poetry shell`. No other dependencies are needed to run pre-commit and the tests. It\'s recommended to use docker (as explained above) if you need the extra (interactive or ocr) dependencies.\n\nInstall pre-commit with `pre-commit install` (don\'t forget you need to prepend all commands with `poetry run ...` if you did not run `poetry shell` first).\n\nRun tests by calling `pytest tests`.\n\n## Dataset and benchmark paper\nThe dataset, the benchmark tasks and the evaluation criteria are described in detail in the [dataset paper](https://arxiv.org/abs/2302.05658). To cite the dataset, please use the following BibTeX entry:\n```\n@misc{simsa2023docile,\n    title={{DocILE} Benchmark for Document Information Localization and Extraction},\n    author={{\\v{S}}imsa, {\\v{S}}t{\\v{e}}p{\\\'a}n and {\\v{S}}ulc, Milan and U{\\v{r}}i{\\v{c}}{\\\'a}{\\v{r}}, Michal and Patel, Yash and Hamdi, Ahmed and Koci{\\\'a}n, Mat{\\v{e}}j and Skalick{\\`y}, Maty{\\\'a}{\\v{s}} and Matas, Ji{\\v{r}}{\\\'\\i} and Doucet, Antoine and Coustaty, Micka{\\"e}l and Karatzas, Dimosthenis},\n    url = {https://arxiv.org/abs/2302.05658},\n    journal={arXiv preprint arXiv:2302.05658},\n    year={2023}\n}\n```\n',
    'author': 'Stepan Simsa',
    'author_email': 'stepan.simsa@rossum.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://docile.rossum.ai/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8.2,<4.0.0',
}


setup(**setup_kwargs)
